package br.unb.cic;

import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;
import fi.hut.tcm.spki.format.*;
import fi.hut.tcm.spki.impl.Tag;
import java.security.*;
import java.io.*;
import java.security.spec.*;
import java.security.KeyFactory;
import java.math.BigInteger;


/**

  A classe Tool realiza operaes auxiliares em certificados SPKI/SDSI no formato cannico. Dentre
  estas esto a obteno das chaves pblicas RSA do emissor (Issuer) e do sujeito (Subject).

   */

public class Tool {


  public static PublicKey getIssuerPublicKeyRSA(InputStream in) throws Exception {

        byte[] buf = new byte[in.available()];
	in.read(buf);
	CanonicalSpki canonical = new CanonicalSpki();
	canonical.putBuffer(buf);
	SpkiParserVisitor p = new  SpkiParserVisitor();
        GenericFormat g = p.run(canonical);

        Sig assinatura = new Sig();
        assinatura = (Sig)g.get(1);

        PubKey pub = new PubKey();
        pub = (PubKey)assinatura.get(1);


        SExpr sexprExpo = (SExpr)pub.get(1);
        String strExpoente =sexprExpo.get(1).toString(); // String 65537

        SExpr sexprMod = (SExpr)pub.get(2);
        String strModulo = sexprMod.get(1).toString(); // Modulo 9749675364356346...

        RSAPublicKeySpec pubkeyspec = new RSAPublicKeySpec(new BigInteger(strModulo),new BigInteger(strExpoente));
        KeyFactory keyfact = KeyFactory.getInstance("RSA");
        PublicKey ChavePublicaEmissorRSA = keyfact.generatePublic(pubkeyspec);
        return ChavePublicaEmissorRSA;


  }

  public static PublicKey getSubjectPublicKeyRSA(InputStream in) throws Exception {

        byte[] buf = new byte[in.available()];
	in.read(buf);
	CanonicalSpki canonical = new CanonicalSpki();
	canonical.putBuffer(buf);
	SpkiParserVisitor p = new  SpkiParserVisitor();
        GenericFormat g = p.run(canonical);

        Cert cer = new Cert();
        cer = (Cert)g.get(0);

        Subject sujeito = new Subject();
        sujeito = (Subject)cer.get(1);

        String strSujeito = sujeito.toString();
        int cont = strSujeito.indexOf("(e ")+3;
        StringBuffer strbuffer = new StringBuffer();
        for (int i = cont; i < strSujeito.length(); i++ ) {
            if (String.valueOf( strSujeito.charAt(i) ).equals(")") )
              break;
            else
              strbuffer.append(strSujeito.charAt(i));
        }
        String strExpoente = strbuffer.toString();
        String strModulo = strSujeito.substring(strSujeito.indexOf("(n ")+3, (strSujeito.length())-3);


        RSAPublicKeySpec pubkeyspec = new RSAPublicKeySpec(new BigInteger(strModulo),new BigInteger(strExpoente));
        KeyFactory keyfact = KeyFactory.getInstance("RSA");
        PublicKey ChavePublicaSujeitoRSA = keyfact.generatePublic(pubkeyspec);
        return ChavePublicaSujeitoRSA;
  }


  public static String getAuthorization(InputStream in) throws Exception {

        byte[] buf = new byte[in.available()];
	in.read(buf);
	CanonicalSpki canonical = new CanonicalSpki();
	canonical.putBuffer(buf);
	SpkiParserVisitor p = new  SpkiParserVisitor();
        GenericFormat g = p.run(canonical);

        Cert cer = new Cert();
        cer = (Cert)g.get(0);

        Tag t = new Tag();
        t = (Tag)cer.get(3);

        TagExpr texpr = new TagExpr();
        texpr = (TagExpr)t.get(0);

        TagSimple tsimple = new TagSimple();
        tsimple =  (TagSimple)texpr.get(0);

        ByteString bstring = new ByteString();
        bstring = (ByteString)tsimple.get(0);

        String strAuthorization = bstring.toString();

        return strAuthorization;

  }


}


