
package br.unb.cic;


import java.security.*;
import java.security.cert.*;
import java.io.*;
import fi.hut.tcm.spki.format.*;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.impl.*;


public class MakeCertificate {

  public static void main(String[] args) throws Exception {

    Security.addProvider(new SPKIProvider());


    Provider prov = Security.getProvider("TeSSA.tcm.hut.fi SPKIProvider");
    System.out.println("Provider name: " + prov.getName());
    System.out.println("Provider version: " + prov.getVersion());
    System.out.println("Provider info: " + prov.getInfo());

    System.out.println("* Making CertificateFactory...");
    CertificateFactory certificatefactory = CertificateFactory.getInstance("SPKI");


    System.out.println("* Making Keys...");
    KeyPairGenerator keygen = KeyPairGenerator.getInstance("RSA");
    keygen.initialize(1024);
    KeyPair kp1 = keygen.genKeyPair();
    KeyPair kp2 = keygen.genKeyPair();

    TagSimple tagsimple = new TagSimple(); //1. TagSimple
    ByteString bytestring = new ByteString("Client_Authorizations");
    System.out.println(tagsimple.check(bytestring));
    tagsimple.add(bytestring);

    TagExpr tagexpr = new TagExpr(); //2. TagExpr
    System.out.println(tagexpr.check(tagsimple));
    tagexpr.add(tagsimple);


    fi.hut.tcm.spki.impl.Tag tag = new fi.hut.tcm.spki.impl.Tag(); //3. Tag
    System.out.println(tag.check(tagexpr));
    tag.add(tagexpr);

    System.out.println("* Making Certificate...");

    SpkiCertFactory spkicertfactory = new SpkiCertFactory();
    SPKICertificate cert = spkicertfactory.makeCert(kp1, kp2.getPublic(), false, tag);


    System.out.println("* Writing Certificate...");
    FileOutputStream fos = new FileOutputStream("Certificate.spki");
    fos.write(cert.getEncoded());
    fos.close();


    System.out.println("* Reading Certificate...");
    FileInputStream fis = new FileInputStream("Certificate.spki");
    SPKICertificate certii = (SPKICertificate)certificatefactory.generateCertificate(fis);
    System.out.println(certii.getSubject());

    certii.verify(kp1.getPublic());

    if (certii.verifica(kp1.getPublic()))
        System.out.println("Certificate OK !\n");


      //Use java -Dfile.encoding=ISO8859_1 <classname>
     System.out.println("Java Virtual Machine Encode: "+System.getProperty("file.encoding"));


  }

}



