/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  CanonicalSpki
  Package: format
  File:    CanonicalSpki.java
*/

package fi.hut.tcm.spki.format;
import java.util.HashMap;
import java.math.BigInteger;
import java.io.ByteArrayOutputStream;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.format.*;
import fi.hut.tcm.spki.impl.*;

/**
  CanonicalSpki
  
  The SPKI canonical (binary) format.
*/
public class CanonicalSpki
  extends Object
  implements Format {

  /**
    Initial capacity of our buffer.
  */
  private static final int DEFAULT_EMPTY_CAPACITY = 512;
  
  /**
  */
  private ByteArrayOutputStream os = new ByteArrayOutputStream(DEFAULT_EMPTY_CAPACITY);
  
  /**
  */
  public static HashMap parseassociations;
  
  /**
  */
  public static HashMap makeassociations;
  
  /**
  */
  public CanonicalSpki() {
  
  //+ CanonicalSpki::CanonicalSpki( )
      // Initializations of name-class associations.
      // These associations are used as bidirectional so Class collisions
      // should be avoided.
      parseassociations = new HashMap();
      parseassociations.put("sequence", (new Sequence()).getClass());
      parseassociations.put("cert", (new Cert()).getClass());
      parseassociations.put("hash", (new Hash()).getClass());
      parseassociations.put("issuer", (new Issuer()).getClass());
      parseassociations.put("subject", (new Subject()).getClass());
      parseassociations.put("do", (new GeneralOp()).getClass());
      parseassociations.put("name", (new Name()).getClass());
      parseassociations.put("keyholder", (new Keyholder()).getClass());
      parseassociations.put("k-of-n", (new ThresholdSubj()).getClass());
      parseassociations.put("secret-key", (new SecKey()).getClass());
      parseassociations.put("public-key", (new PubKey()).getClass());
      parseassociations.put("online", (new fi.hut.tcm.spki.impl.OnlineTest()).getClass());
      parseassociations.put("object-hash", (new ObjHash()).getClass());
      parseassociations.put("tag", (new fi.hut.tcm.spki.impl.Tag()).getClass());
      parseassociations.put("signature", (new Sig()).getClass());
      parseassociations.put("not-before", (new NotBefore()).getClass());
      parseassociations.put("not-after", (new NotAfter()).getClass());
      parseassociations.put("propagate", (new Deleg()).getClass());
      parseassociations.put("comment", (new Comment()).getClass());
      parseassociations.put("version", (new SkipSExpr()).getClass());
      parseassociations.put("display", (new SkipSExpr()).getClass());
      parseassociations.put("issuer-info", (new SkipSExpr()).getClass());
      parseassociations.put("subject-info", (new SkipSExpr()).getClass());

      makeassociations = new HashMap();
      makeassociations.put((new Sequence()).getClass(), "sequence");
      makeassociations.put((new Cert()).getClass(), "cert");
      makeassociations.put((new Hash()).getClass(), "hash");
      makeassociations.put((new Issuer()).getClass(), "issuer");
      makeassociations.put((new Subject()).getClass(), "subject");
      makeassociations.put((new GeneralOp()).getClass(), "do");
      makeassociations.put((new Name()).getClass(), "name");
      makeassociations.put((new Keyholder()).getClass(), "keyholder");
      makeassociations.put((new ThresholdSubj()).getClass(), "k-of-n");
      makeassociations.put((new SecKey()).getClass(), "secret-key");
      makeassociations.put((new PubKey()).getClass(), "public-key");
      makeassociations.put((new fi.hut.tcm.spki.impl.OnlineTest()).getClass(), "online");
      makeassociations.put((new ObjHash()).getClass(), "object-hash");
      makeassociations.put((new fi.hut.tcm.spki.impl.Tag()).getClass(), "tag");
      makeassociations.put((new Sig()).getClass(), "signature");
      makeassociations.put((new NotBefore()).getClass(), "not-before");
      makeassociations.put((new NotAfter()).getClass(), "not-after");
      makeassociations.put((new Deleg()).getClass(), "propagate");
      makeassociations.put((new Comment()).getClass(), "comment");
      makeassociations.put((new SkipSExpr()).getClass(), "version");
      makeassociations.put((new SkipSExpr()).getClass(), "display");
      makeassociations.put((new SkipSExpr()).getClass(), "issuer-info");
      makeassociations.put((new SkipSExpr()).getClass(), "subject-info");
  //- CanonicalSpki::CanonicalSpki( )
  }
  
  /**
  */
  public byte[] getBuffer() {
  
  //+ CanonicalSpki::getBuffer( )
    return os.toByteArray();
  //- CanonicalSpki::getBuffer( )
  }
  
  /**
  */
  public void putBuffer(byte[] buf) {
  
  //+ CanonicalSpki::putBuffer(byte[])
    os.reset();
    os.write(buf, 0, buf.length);
  //- CanonicalSpki::putBuffer(byte[])
  }
  
  /**
    Append bytes without a length indicator.
  */
  public void appendBytes(String str) {
  
  //+ CanonicalSpki::appendBytes(String)
    byte[] b = str.getBytes();
    
    os.write(b, 0, b.length);
  //- CanonicalSpki::appendBytes(String)
  }
  
  /**
    Append a byte string and it's length indicator.
  */
  public void appendString(String str) {
  
  //+ CanonicalSpki::appendString(String)
    appendBytes(String.valueOf(str.length()));
    appendBytes(":");
    appendBytes(str);
  //- CanonicalSpki::appendString(String)
  }
  
  /**
  */
  public void appendNumber(BigInteger number) {
  
  //+ CanonicalSpki::appendNumber(BigInteger)
    appendBytes(String.valueOf(number.toByteArray().length));
    appendBytes(":");
    appendBytes(new String(number.toByteArray()));
  //- CanonicalSpki::appendNumber(BigInteger)
  }
  
  /**
  */
  public String toString() {
  
  //+ CanonicalSpki::toString( )
    // XXX This should be Base64 encoded, shouldn't it?
    return new String(getBuffer());
  //- CanonicalSpki::toString( )
  }
  
} // CanonicalSpki

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
