/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  PubKey
  Package: impl
  File:    PubKey.java
  Modificado por Cristian F. Souza
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.format.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import java.math.BigInteger;

/**
  PubKey

  Public Key class. Contains a PubSigAlgId followed by an arbitrary number
  of SExpr:s followed by on optional Uri. SExpr:s contain the key data and
  the Uri is a pointer to additional key info.
*/
public class PubKey
  extends Composite
  implements SpkiPrincipal, Key, SeqEnt, PublicKey, java.security.Key, java.io.Serializable {

  /**
  */
  public PubKey() {

  //+ PubKey::PubKey( )
    super();
  //- PubKey::PubKey( )
  }

  /**
  */
  public PubKey(PublicKey key) {

  //+ PubKey::PubKey(PublicKey)
    super();

    String alg = key.getAlgorithm();
    PubSigAlgId id = new PubSigAlgId();
    SExpr expr = new SExpr();
    SPart subexpr = new SPart();
    if (alg.equals("DSA")) {
      id.add(new ByteString("dsa-sha1"));
      add(id);

      expr.add(new ByteString("y"));
      subexpr.add(new ByteString(((DSAPublicKey) key).getY().toString()));
      expr.add(subexpr);
      add(expr);

      expr = new SExpr();
      subexpr = new SPart();
      expr.add(new ByteString("p"));
      subexpr.add(new ByteString(((DSAKey) key).getParams().getP().toString()));
      expr.add(subexpr);
      add(expr);

      expr = new SExpr();
      subexpr = new SPart();
      expr.add(new ByteString("q"));
      subexpr.add(new ByteString(((DSAKey) key).getParams().getQ().toString()));
      expr.add(subexpr);
      add(expr);
      expr = new SExpr();
      subexpr = new SPart();
      expr.add(new ByteString("g"));
      subexpr.add(new ByteString(((DSAKey) key).getParams().getG().toString()));
      expr.add(subexpr);
      add(expr);
    }
    else if (alg.equals("RSA")) {
      id.add(new ByteString("rsa-pkcs1-sha1"));
      add(id);

      expr.add(new ByteString("e"));
      subexpr.add(new ByteString(((RSAPublicKey) key).getPublicExponent().toString()));
      expr.add(subexpr);
      add(expr);
      expr = new SExpr();
      subexpr = new SPart();
      expr.add(new ByteString("n"));
      subexpr.add(new ByteString(((RSAPublicKey) key).getModulus().toString()));
      expr.add(subexpr);
      add(expr);
    }
    // XXX Add the elliptic and hyperelliptic curves here!

  //- PubKey::PubKey(PublicKey)
  }

  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {

  //+ PubKey::check(Component)
      if (!(addition instanceof PubSigAlgId) &&
	  !(addition instanceof SExpr) &&
	  !(addition instanceof Uri))
	return false;
      if (children.size() == 0)
	if (addition instanceof PubSigAlgId)
	  return true;
	else
	  throw new IllegalStateException();
      if ((children.size() != 0) &&
	  (addition instanceof PubSigAlgId))
	throw new IllegalStateException();
      if ((children.size() == 1) &&
	  (addition instanceof Uri))
	throw new IllegalStateException();
      if (get(children.size() - 1) instanceof Uri)
	throw new IllegalStateException();
      return true;
  //- PubKey::check(Component)
  }

  /**
  */
  public void accept(Visitor visitor) {

  //+ PubKey::accept(Visitor)
      visitor.at(this);
  //- PubKey::accept(Visitor)
  }

  /**
  */
  public String getAlgorithm() {

  //+ PubKey::getAlgorithm( )
    return getPublicKey().getAlgorithm();
  //- PubKey::getAlgorithm( )
  }

  /**
  */
  public byte[] getEncoded() {

  //+ PubKey::getEncoded( )
    return getPublicKey().getEncoded();
  //- PubKey::getEncoded( )
  }

  /**
  */
  public String getFormat() {

  //+ PubKey::getFormat( )
    return getPublicKey().getFormat();
  //- PubKey::getFormat( )
  }

  /**
  */
  public PublicKey getPublicKey() {

  //+ PubKey::getPublicKey( )
    PubSigAlgId id = (PubSigAlgId) get(new PubSigAlgId());
      if ((id.get(0) instanceof ByteString) &&
	((ByteString) id.get(new ByteString())).getValueString().equals("rsa-pkcs1-sha1")) {


      BigInteger expoente = new BigInteger(((Composite) get(1)).get(new SPart()).toString());

      BigInteger modulo = new BigInteger(((Composite) get(2)).get(new SPart()).toString());


      try {

        RSAPublicKeySpec spec = new RSAPublicKeySpec(modulo,expoente);
        KeyFactory factory = KeyFactory.getInstance("RSA");
	return factory.generatePublic(spec);

      } catch (NoSuchAlgorithmException e) {
	// Not reached, because JDK 1.2 includes the DSA implementation
	System.err.println("Unknown algorithm: " + e.toString());
      } catch (InvalidKeySpecException e) {
	System.err.println("Invalid KeySpec: " + e.toString());
      }
    }
    return null;
  //- PubKey::getPublicKey( )
  }

  /**
  */
  public String toString() {

  //+ PubKey::toString( )
      String result;

      result = "(public-key ";
      for (int i = 0; i < size(); i++) {
	result += children.elementAt(i).toString();
	if (i < (size() - 1))
	  result += " ";
      }
      result += ")";
      return result;
  //- PubKey::toString( )
  }

} // PubKey

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
