/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  SpkiMakerVisitor
  Package: impl
  File:    SpkiMakerVisitor.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import fi.hut.tcm.spki.format.*;
import fi.hut.tcm.spki.impl.*;
import java.util.*;
import java.math.BigInteger;

/**
  SpkiMakerVisitor

  Traverses GenericFormat and creates a SPKI Canonical Format on the way.
*/
public class SpkiMakerVisitor
  extends Object
  implements ComponentVisitor {

  /**
  */
  private static final long version = 0;

  /**
  */
  private CanonicalSpki spooky = new CanonicalSpki();

  /**
  */
  public SpkiMakerVisitor() {

  //+ SpkiMakerVisitor::SpkiMakerVisitor( )
    super();
  //- SpkiMakerVisitor::SpkiMakerVisitor( )
  }

  /**
    Does the job. Even if CanonicalSpki is successfully returned, it does
    not mean that it is meaningful. The maker of GenericFormat is
    responsible for the semantic content.
  */
  public CanonicalSpki run(Component startAt)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::run(Component)
      try {
	startAt.accept(this);
      } catch (ArrayIndexOutOfBoundsException e) { // From Component::get(int)
	throw new IllegalArgumentException();
      }
      return spooky;
  //- SpkiMakerVisitor::run(Component)
  }

  /**
  */
  public void att(Sequence node) {

  //+ SpkiMakerVisitor::att(Sequence)
      doAllInOrder(node, 1, 0);
  //- SpkiMakerVisitor::att(Sequence)
  }

  /**
  */
  public void att(SPart node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(SPart)
      // Get may throw ArrayIndexOutOfBoundsException, which run() catches.
      node.get(0).accept(this);
      try {
	node.get(1);
      } catch (ArrayIndexOutOfBoundsException e) {
	return;
      }
      throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(SPart)
  }

  /**
  */
  public void att(Hash node) {

  //+ SpkiMakerVisitor::att(Hash)
      doAllInOrder(node, 2, 3);
  //- SpkiMakerVisitor::att(Hash)
  }

  /**
  */
  public void att(SExpr node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(SExpr)
      if (node.size() == 0)
	throw new IllegalArgumentException();
      spooky.appendBytes("(");
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(SExpr)
  }

  /**
  */
  public void att(HashAlgName node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(HashAlgName)
      Component s = node.get(0);

      if (s instanceof Uri)
	s.accept(this);
      else {
	String value = ((ByteString) s).getValueString();
	if (value.equals("md5") ||
	    value.equals("sha1"))
	  spooky.appendString(value);
	else
	  throw new IllegalArgumentException();
      }
      try {
	node.get(1);
      } catch (ArrayIndexOutOfBoundsException e) {
	return;
      }
      throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(HashAlgName)
  }

  /**
  */
  public void att(Issuer node) {

  //+ SpkiMakerVisitor::att(Issuer)
      doAllInOrder(node, 1, 1);
      // XXX some additional constraints on the child should be stated.
      // Name should contain only one namestep
      // Key should not be a SecKey
      // XXX No children means "self"
  //- SpkiMakerVisitor::att(Issuer)
  }

  /**
  */
  public void att(Subject node) {

  //+ SpkiMakerVisitor::att(Subject)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(Subject)
  }

  /**
  */
  public void att(GeneralOp node) {

  //+ SpkiMakerVisitor::att(GeneralOp)
      doAllInOrder(node, 1, 0);
  //- SpkiMakerVisitor::att(GeneralOp)
  }

  /**
  */
  public void att(HashOp node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(HashOp)
      spooky.appendBytes("(");
      spooky.appendString("do");
      spooky.appendString("hash");
      node.get(0).accept(this);
      spooky.appendBytes(")");
      try {
	node.get(1);
      } catch (ArrayIndexOutOfBoundsException e) {
	return;
      }
      throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(HashOp)
  }

  /**
  */
  public void att(Name node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(Name)
      if ((doAllInOrder(node, 1, 0) == 1) &&
	  (node.get(0) instanceof Key))
	throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(Name)
  }

  /**
  */
  public void att(Keyholder node) {

  //+ SpkiMakerVisitor::att(Keyholder)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(Keyholder)
  }

  /**
  */
  public void att(ThresholdSubj node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(ThresholdSubj)
      if (node.size() < 2)
	throw new IllegalArgumentException();
      spooky.appendBytes("(");
      spooky.appendString
	((String) spooky.makeassociations.get(node.getClass()));
      Iterator e = node.iterator();
      ((Component) e.next()).accept(this);
      spooky.appendNumber
	(BigInteger.valueOf((long) node.size() - 1));
      for (e.next(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(ThresholdSubj)
  }

  /**
  */
  public void att(SecKey node) {

  //+ SpkiMakerVisitor::att(SecKey)
      doAllInOrder(node, 2, 0);
  //- SpkiMakerVisitor::att(SecKey)
  }

  /**
  */
  public void att(PubKey node) {

  //+ SpkiMakerVisitor::att(PubKey)
      doAllInOrder(node, 2, 0);
  //- SpkiMakerVisitor::att(PubKey)
  }

  /**
  */
  public void att(PubSigAlgId node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(PubSigAlgId)
      if (node.get(0) instanceof ByteString) {
	String value = ((ByteString) node.get(0)).getValueString();
	if (!value.equals("rsa-pkcs1-md5") &&
	    !value.equals("rsa-pkcs1-sha1") &&
	    !value.equals("dsa-sha1"))
	  throw new IllegalArgumentException();
      }
      node.get(0).accept(this);
      try {
	node.get(1);
      } catch (ArrayIndexOutOfBoundsException e) {
	return;
      }
      throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(PubSigAlgId)
  }

  /**
  */
  public void att(SecSigAlgId node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(SecSigAlgId)
      if (node.get(0) instanceof ByteString) {
	String value = ((ByteString) node.get(0)).getValueString();
	if (!value.equals("hmac-md5") &&
	    !value.equals("hmac-sha1") &&
	    !value.equals("des-cbc-mac"))
	  throw new IllegalArgumentException();
      }
      node.get(0).accept(this);
      try {
	node.get(1);
      } catch (ArrayIndexOutOfBoundsException e) {
	return;
      }
      throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(SecSigAlgId)
  }

  /**
  */
  public void att(OnlineTest node) {

  //+ SpkiMakerVisitor::att(OnlineTest)
      doAllInOrder(node, 3, 0);
  //- SpkiMakerVisitor::att(OnlineTest)
  }

  /**
  */
  public void att(ObjHash node) {

  //+ SpkiMakerVisitor::att(ObjHash)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(ObjHash)
  }

  /**
  */
  public void att(Tag node) {

  //+ SpkiMakerVisitor::att(Tag)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(Tag)
  }

  /**
  */
  public void att(Sig node) {

  //+ SpkiMakerVisitor::att(Sig)
      doAllInOrder(node, 3, 3);
  //- SpkiMakerVisitor::att(Sig)
  }

  /**
  */
  public void att(Cert node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(Cert)
      if ((node.size() < 4) ||
	  (node.size() > 7))
	throw new IllegalArgumentException();

      spooky.appendBytes("(");
      spooky.appendString
	((String) spooky.makeassociations.get(node.getClass()));
      spooky.appendBytes("(");
      spooky.appendString("version");
      spooky.appendNumber(BigInteger.valueOf(version));
      spooky.appendBytes(")");
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
      // XXX A signature should be added if this is not a sequence
  //- SpkiMakerVisitor::att(Cert)
  }

  /**
  */
  public void att(Valid node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(Valid)
      if (node.size() > 3)
	throw new IllegalArgumentException();

      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
  //- SpkiMakerVisitor::att(Valid)
  }

  /**
  */
  public void att(NotBefore node) {

  //+ SpkiMakerVisitor::att(NotBefore)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(NotBefore)
  }

  /**
  */
  public void att(NotAfter node) {

  //+ SpkiMakerVisitor::att(NotAfter)
      doAllInOrder(node, 1, 1);
  //- SpkiMakerVisitor::att(NotAfter)
  }

  /**
  */
  public void att(Deleg node) {

  //+ SpkiMakerVisitor::att(Deleg)
      if (node.getValueString().equals("t")) {
	spooky.appendBytes("(");
	spooky.appendString
	  ((String) spooky.makeassociations.get(node.getClass()));
	spooky.appendBytes(")");
      }
  //- SpkiMakerVisitor::att(Deleg)
  }

  /**
  */
  public void att(Comment node) {

  //+ SpkiMakerVisitor::att(Comment)
	spooky.appendBytes("(");
	spooky.appendString
	  ((String) spooky.makeassociations.get(node.getClass()));
	att((ByteString) node);
	spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(Comment)
  }

  /**
  */
  public void att(KVal node) {

  //+ SpkiMakerVisitor::att(KVal)
      att((ByteString) node);
  //- SpkiMakerVisitor::att(KVal)
  }

  /**
  */
  public void att(Uri node) {

  //+ SpkiMakerVisitor::att(Uri)
      att((ByteString) node);
  //- SpkiMakerVisitor::att(Uri)
  }

  /**
  */
  public void att(OnlineType node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(OnlineType)
      String s = node.getValueString();

      if (s.equals("crl") || s.equals("reval") || s.equals("one-time"))
	att((ByteString) node);
      else
	throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(OnlineType)
  }

  /**
  */
  public void att(HashValue node) {

  //+ SpkiMakerVisitor::att(HashValue)
      att((ByteString) node);
  //- SpkiMakerVisitor::att(HashValue)
  }

  /**
  */
  public void att(Time node) {

  //+ SpkiMakerVisitor::att(Time)
      att((ByteString) node);
  //- SpkiMakerVisitor::att(Time)
  }

  /**
  */
  public void att(HashOfKey node) {

  //+ SpkiMakerVisitor::att(HashOfKey)
      att((Hash) node);
  //- SpkiMakerVisitor::att(HashOfKey)
  }

  /**
  */
  public void att(SigVal node) {

  //+ SpkiMakerVisitor::att(SigVal)
      att((SPart) node);
  //- SpkiMakerVisitor::att(SigVal)
  }

  /**
  */
  public void att(ByteString node) {

  //+ SpkiMakerVisitor::att(ByteString)
      if (node.getDisplayHintString().length() > 0) {
	spooky.appendBytes("[");
	spooky.appendString(node.getDisplayHintString());
	spooky.appendBytes("]");
      }
      spooky.appendString(node.getValueString());
  //- SpkiMakerVisitor::att(ByteString)
  }

  /**
  */
  public void att(Lte node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(Lte)
      String s = node.getValueString();

      if (s.equals("l") || s.equals("le"))
	att((ByteString) node);
      else
	throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(Lte)
  }

  /**
  */
  public void att(Gte node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(Gte)
      String s = node.getValueString();

      if (s.equals("g") || s.equals("ge"))
	att((ByteString) node);
      else
	throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(Gte)
  }

  /**
  */
  public void att(RangeOrdering node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(RangeOrdering)
      String s = node.getValueString();

      if (s.equals("alpha") ||
	  s.equals("numeric") ||
	  s.equals("time") ||
	  s.equals("binary") ||
	  s.equals("date"))
	att((ByteString) node);
      else
	throw new IllegalArgumentException();
  //- SpkiMakerVisitor::att(RangeOrdering)
  }

  /**
  */
  public void att(TagStar node) {

  //+ SpkiMakerVisitor::att(TagStar)
      spooky.appendBytes("(");
      spooky.appendString("*");
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(TagStar)
  }

  /**
  */
  public void att(UpLim node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(UpLim)
      if ((node.size() < 1) ||
	  (node.size() > 2))
	throw new IllegalArgumentException();

      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
  //- SpkiMakerVisitor::att(UpLim)
  }

  /**
  */
  public void att(LowLim node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(LowLim)
      if ((node.size() < 1) ||
	  (node.size() > 2))
	throw new IllegalArgumentException();

      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
  //- SpkiMakerVisitor::att(LowLim)
  }

  /**
  */
  public void att(TagRange node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(TagRange)
      if ((node.size() < 1) ||
	  (node.size() > 3))
	throw new IllegalArgumentException();

      spooky.appendBytes("(");
      spooky.appendString("*");
      spooky.appendString("range");
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(TagRange)
  }

  /**
  */
  public void att(TagPrefix node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(TagPrefix)
      if ((node.size() != 1))
	throw new IllegalArgumentException();

      spooky.appendBytes("(");
      spooky.appendString("*");
      spooky.appendString("prefix");
      node.get(0).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(TagPrefix)
  }

  /**
  */
  public void att(TagSet node) {

  //+ SpkiMakerVisitor::att(TagSet)
      spooky.appendBytes("(");
      spooky.appendString("*");
      spooky.appendString("set");
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(TagSet)
  }

  /**
  */
  public void att(TagString node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(TagString)
      if ((node.size() != 1))
	throw new IllegalArgumentException();

      node.get(0).accept(this);
  //- SpkiMakerVisitor::att(TagString)
  }

  /**
  */
  public void att(TagSimple node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(TagSimple)
      if (node.size() == 0)
	throw new IllegalArgumentException();
      spooky.appendBytes("(");
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
  //- SpkiMakerVisitor::att(TagSimple)
  }

  /**
  */
  public void att(TagExpr node)
    throws IllegalArgumentException  {

  //+ SpkiMakerVisitor::att(TagExpr)
      if ((node.size() != 1))
	throw new IllegalArgumentException();

      node.get(0).accept(this);
  //- SpkiMakerVisitor::att(TagExpr)
  }

  /**
  */
  public void at(Object node) {

  //+ ComponentVisitor::at(Object)
    if (node.getClass().getName().equals("fi.hut.tcm.spki.ByteString"))
      att((ByteString) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Cert"))
      att((Cert) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Comment"))
      att((Comment) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Deleg"))
      att((Deleg) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.GeneralOp"))
      att((GeneralOp) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Gte"))
      att((Gte) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Hash"))
      att((Hash) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.HashAlgName"))
      att((HashAlgName) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.HashOfKey"))
      att((HashOfKey) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.HashOp"))
      att((HashOp) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.HashValue"))
      att((HashValue) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Issuer"))
      att((Issuer) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Keyholder"))
      att((Keyholder) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.KVal"))
      att((KVal) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.LowLim"))
      att((LowLim) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Lte"))
      att((Lte) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Name"))
      att((Name) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.NotAfter"))
      att((NotAfter) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.NotBefore"))
      att((NotBefore) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.ObjHash"))
      att((ObjHash) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.OnlineTest"))
      att((OnlineTest) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.OnlineType"))
      att((OnlineType) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.PubKey"))
      att((PubKey) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.PubSigAlgId"))
      att((PubSigAlgId) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.RangeOrdering"))
      att((RangeOrdering) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.SecKey"))
      att((SecKey) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.SecSigAlgId"))
      att((SecSigAlgId) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.format.Sequence"))
      att((Sequence) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.SExpr"))
      att((SExpr) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Sig"))
      att((Sig) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.SigVal"))
      att((SigVal) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.SPart"))
      att((SPart) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Subject"))
      att((Subject) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Tag"))
      att((Tag) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagExpr"))
      att((TagExpr) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagPrefix"))
      att((TagPrefix) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagRange"))
      att((TagRange) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagSet"))
      att((TagSet) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagSimple"))
      att((TagSimple) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagStar"))
      att((TagStar) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.TagString"))
      att((TagString) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.ThresholdSubj"))
      att((ThresholdSubj) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Time"))
      att((Time) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.UpLim"))
      att((UpLim) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Uri"))
      att((Uri) node);
    if (node.getClass().getName().equals("fi.hut.tcm.spki.impl.Valid"))
      att((Valid) node);
  //- ComponentVisitor::at(Object)
  }

  /**
    Goes through the children of the node and writes the result to
    CanonicalSpki. Returns the number of children done. Throws
    IllegalArgumentException if the number of children was not in the range
     defined by min and max. If max is zero, there is no upper limit. Min
    and max belong to the successful values. Throws NoSuchElementException
    if class identifier string was not found in CanonicalSpki.associations.
  */
  private int doAllInOrder(Composite node, int min, int max)
    throws IllegalArgumentException,NoSuchElementException  {

  //+ SpkiMakerVisitor::doAllInOrder(Composite, int, int)
      int i = node.size();

      if (((max > 0) && (i > max)) || (i < min))
	throw new IllegalArgumentException();
      spooky.appendBytes("(");
      spooky.appendString
	((String) spooky.makeassociations.get(node.getClass()));
      for (Iterator e = node.iterator(); e.hasNext(); )
	((Component) e.next()).accept(this);
      spooky.appendBytes(")");
      return i;
  //- SpkiMakerVisitor::doAllInOrder(Composite, int, int)
  }

} // SpkiMakerVisitor

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
