/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Subject
  Package: impl
  File:    Subject.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.security.*;

/**
  Subject
  
  The information about certificate subject or subjects. Contains one 
  instance of a class Name, Key, Keyholder, ThresholdSubj OR ObjHash.
*/
public class Subject
  extends Composite {

  /**
  */
  public Subject() {
  
  //+ Subject::Subject( )
    super();
  //- Subject::Subject( )
  }
  
  /**
  */
  public Subject(PublicKey key) {
  
  //+ Subject::Subject(PublicKey)
    super();
    try {
      add(new PubKey(key));
    } catch (Exception e) {
      // Not reached.
    }
  //- Subject::Subject(PublicKey)
  }
  
  /**
    Makes an obj-hash subject of the objectbytes.
  */
  public Subject(byte[] objectbytes) {
  
  //+ Subject::Subject(byte[])
    super();
    try {
      ObjHash obj = new ObjHash(); 
      Hash h = new Hash();
      
      HashAlgName algname = new HashAlgName();
      algname.add(new ByteString("sha1"));
      h.add(algname);
      
      MessageDigest md = MessageDigest.getInstance("SHA");
      byte[] digest = md.digest(objectbytes);
      //      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      //      sun.misc.CharacterEncoder coder64 = new sun.misc.BASE64Encoder();
      //      coder64.encodeBuffer(digest, baos);
      //      HashValue hval = new HashValue("|" + baos.toString() + "|");
      HashValue hval = new HashValue(digest);
      h.add(hval);
      
      obj.add(h);
      add(obj);
    } catch (NoSuchAlgorithmException e) {
      // FIX
    }
  //- Subject::Subject(byte[])
  }
  
  /**
    Makes an obj-hash subject of the objectbytes.
  */
  public Subject(byte[] objectbytes, String hashUri) {
  
  //+ Subject::Subject(byte[], String)
    super();
    try {
      ObjHash obj = new ObjHash(); 
      Hash h = new Hash();
      Uri u = new Uri(hashUri);
      
      HashAlgName algname = new HashAlgName();
      algname.add(new ByteString("sha1"));
      h.add(algname);
      
      MessageDigest md = MessageDigest.getInstance("SHA");
      byte[] digest = md.digest(objectbytes);
      //      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      //      sun.misc.CharacterEncoder coder64 = new sun.misc.BASE64Encoder();
      //      coder64.encodeBuffer(digest, baos);
      //      HashValue hval = new HashValue("|" + baos.toString() + "|");
      HashValue hval = new HashValue(digest);
      h.add(hval);
      h.add(u);
      
      obj.add(h);
      add(obj);
    } catch (NoSuchAlgorithmException e) {
      // FIX
    }
  //- Subject::Subject(byte[], String)
  }
  
  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ Subject::check(Component)
      if (!(addition instanceof Name) &&
	  !(addition instanceof Key) &&
	  !(addition instanceof Keyholder) &&
	  !(addition instanceof ThresholdSubj) &&
	  !(addition instanceof ObjHash))
	return false;
      if (children.size() == 0)
	return true;
      throw new IllegalStateException();
  //- Subject::check(Component)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Subject::accept(Visitor)
      visitor.at(this);
  //- Subject::accept(Visitor)
  }
  
  /**
  */
  public String toString() {
  
  //+ Subject::toString( )
      String result;

      result = "(subject ";
      for (int i = 0; i < size(); i++) {
	result += children.elementAt(i).toString();
	if (i < (size() - 1))
	  result += " ";
      }
      result += ")";
      return result;
  //- Subject::toString( )
  }
  
} // Subject

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
