/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Time
  Package: impl
  File:    Time.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

/**
  Time

  A time-field. This is an ASCII byte string of the form: YYYY-MM-DD_HH:MM:
  SS, always UTC.
*/
class Time
  extends ByteString {

  /**
  */
  private Date date;

  /**
  */
  private SimpleDateFormat format;

  /**
  */
  public Time() {

  //+ Time::Time( )
    super();
    format = new SimpleDateFormat("yyyy-MM-dd_kk:mm:ss");
    try {
      setValue(format.format(new Date()));
    } catch (Exception e) {
      System.err.println(e); // Never reached.
    }
  //- Time::Time( )
  }

  /**
  */
  public Time(Date date) {

  //+ Time::Time(Date)
    super();
    try {
      setValue(format.format(date));
    } catch (Exception e) {
      System.err.println(e); // Never reached.
    }
  //- Time::Time(Date)
  }

  /**
  */
  public Time(byte[] value) {

  //+ Time::Time(byte[])
    super();
    try {
      setValue(new String(value));
    } catch (Exception e) {
      System.err.println("Cannot create time with value " +
			 new String(value) + "\n" + e.toString());
    }
  //- Time::Time(byte[])
  }

  /**
  */
  public Time(String value) {

  //+ Time::Time(String)
    super();
    try {
      setValue(value);
    } catch (Exception e) {
      System.err.println("Cannot create time with value " +
			 value + "\n" + e.toString());
    }
  //- Time::Time(String)
  }

  /**
  */
  public Time(byte[] value, byte[] display) {

  //+ Time::Time(byte[], byte[])
    super();
    ParsePosition pos = new ParsePosition(0);
    try {
      Date d = format.parse((new String(value)), pos);
      date = d;
      super.setValue(format.format(d));
      super.setDisplayHint(display);
    } catch (Exception e) {
      System.err.println("Cannot create a time with value " + value + e.toString());
    }
  //- Time::Time(byte[], byte[])
  }

  /**
  */
  public Time(String value, String display) {

  //+ Time::Time(String, String)
    super();
    ParsePosition pos = new ParsePosition(0);
    try {
      Date d = format.parse(value, pos);
      date = d;
      super.setValue(format.format(d));
      super.setDisplayHint(display);
    } catch (Exception e) {
      System.err.println("Cannot create a time with value " + value + e.toString());
    }
  //- Time::Time(String, String)
  }

  /**
  */
  public String toString() {

  //+ Time::toString( )
    return super.toString();
  //- Time::toString( )
  }

  /**
  */
  public boolean equals(Object other) {

  //+ Time::equals(Object)
    if ((other instanceof Time) &&
	(((Time) other).getValueString().equals(this.getValueString())) &&
	(((Time) other).getDisplayHintString().equals(this.getDisplayHintString())))
      return true;
    else
      return false;
  //- Time::equals(Object)
  }

  /**
  */
  public void setValue(byte[] string)
    throws SPKIException  {

  //+ Time::setValue(byte[])
    ParsePosition pos = new ParsePosition(0);
    try {
      Date d = format.parse((new String(string)), pos);
      date = d;
      super.setValue(string);
    } catch (Exception e) {
      throw new SPKIException(e.toString());
    }
  //- Time::setValue(byte[])
  }

  /**
  */
  public void setValue(String string)
    throws SPKIException  {

  //+ Time::setValue(String)
    ParsePosition pos = new ParsePosition(0);
    try {
      Date d = format.parse(string, pos);
      date = d;
      super.setValue(string);
    } catch (Exception e) {
      throw new SPKIException(e.toString());
    }
  //- Time::setValue(String)
  }

  /**
  */
  public void accept(Visitor visitor) {

  //+ Time::accept(Visitor)
    visitor.at(this);
  //- Time::accept(Visitor)
  }

  /**
  */
  public Date getDate() {

  //+ Time::getDate( )
    return date;
  //- Time::getDate( )
  }

} // Time

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
