/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  Valid
  Package: impl
  File:    Valid.java
*/

package fi.hut.tcm.spki.impl;
import fi.hut.tcm.spki.*;
import java.util.Date;

/**
  Valid
  
  Valid contains details that define when the certificate is valid. The 
  components are NotBefore, NotAfter and OnlineTest, in that order. Each 
  field is optional and could be omitted.
*/
public class Valid
  extends Composite
  implements Validity, fi.hut.tcm.spki.Composite, Component {

  /**
  */
  public Valid() {
  
  //+ Valid::Valid( )
    super();
  //- Valid::Valid( )
  }
  
  /**
  */
  public Valid(Date notAfter) {
  
  //+ Valid::Valid(Date)
    super();
    NotAfter na = new NotAfter();
    na.add(new Time(notAfter));
    add(na);
  //- Valid::Valid(Date)
  }
  
  /**
  */
  public Valid(Date notBefore, Date notAfter) {
  
  //+ Valid::Valid(Date, Date)
    super();
    NotBefore nb = new NotBefore();
    nb.add(new Time(notBefore));
    add(nb);
    NotAfter na = new NotAfter();
    na.add(new Time(notAfter));
    add(na);
  //- Valid::Valid(Date, Date)
  }
  
  /**
  */
  public Valid(OnlineTest online) {
  
  //+ Valid::Valid(OnlineTest)
    super();
    add(online);
  //- Valid::Valid(OnlineTest)
  }
  
  /**
  */
  public Valid(Date notAfter, OnlineTest online) {
  
  //+ Valid::Valid(Date, OnlineTest)
    super();
    NotAfter na = new NotAfter();
    na.add(new Time(notAfter));
    add(na);
    add(online);
  //- Valid::Valid(Date, OnlineTest)
  }
  
  /**
  */
  public Valid(Date notBefore, Date notAfter, OnlineTest online) {
  
  //+ Valid::Valid(Date, Date, OnlineTest)
    super();
    NotBefore nb = new NotBefore();
    nb.add(new Time(notBefore));
    add(nb);
    NotAfter na = new NotAfter();
    na.add(new Time(notAfter));
    add(na);
    add(online);
  //- Valid::Valid(Date, Date, OnlineTest)
  }
  
  /**
  */
  public boolean check(Component addition)
    throws IllegalStateException  {
  
  //+ Valid::check(Component)
      if ((addition instanceof NotBefore) && 
	  (children.size() == 0))
	return true;
      if ((addition instanceof NotAfter) && 
	  ((children.size() == 0) ||
	   (children.size() == 1)))
	return true;
      if ((addition instanceof OnlineTest) && 
	  ((children.size() == 0) || 
	   (children.size() == 1) || 
	   (children.size() == 2)))
	return true;
      else
	return false;
  //- Valid::check(Component)
  }
  
  /**
  */
  public void accept(Visitor visitor) {
  
  //+ Valid::accept(Visitor)
    visitor.at(this);
  //- Valid::accept(Visitor)
  }

  /**
   */
  public Date getNotBefore() {

  //+ Valid::getNotBefore( )
    NotBefore nb = (NotBefore) get(new NotBefore());
    if (nb != null) 
      return ((Time) nb.get(new Time())).getDate();
    else
      return null;
  //- Valid::getNotBefore( )
  }

  /**
   */
  public Date getNotAfter() {

  //+ Valid::getNotAfter( )
    NotAfter na = (NotAfter) get(new NotAfter());
    if (na != null) 
      return ((Time) na.get(new Time())).getDate();
    else
      return null;
  //- Valid::getNotAfter( )
  }

  /**
   */
  public fi.hut.tcm.spki.OnlineTest getOnlineTest() {

  //+ Valid::getOnlineTest( )
    return (OnlineTest) get(new OnlineTest());
  //- Valid::getOnlineTest( )
  }

  /**
   */
  public boolean test() {

  //+ Valid::test( )
    Date now = new Date();
    NotBefore nb = (NotBefore) get(new NotBefore());
    NotAfter na =  (NotAfter) get(new NotAfter());
    OnlineTest ot = (OnlineTest) get(new OnlineTest());
    if ((nb != null) && now.after(((Time) nb.get(new Time())).getDate()) && 
	(na != null) && now.before(((Time) na.get(new Time())).getDate()) &&
	(ot != null) && ot.test())
      return true;
    else 
      return false;
  //- Valid::test( )
  }
  
  /**
   */
  public String toString() {

  //+ Valid::toString( )
    String result = new String();
    if (get(new NotBefore()) != null)
      result = new String(result + (get(new NotBefore())).toString());
    if (get(new NotAfter()) != null)
      result = new String(result + (get(new NotAfter())).toString());
    if (get(new OnlineTest()) != null)
      result = new String(result + (get(new OnlineTest())).toString());
    return result;
  //- Valid::toString( )
  }
  
} // Valid

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
